#include <cstdio>
#include <queue>

using namespace std;

const int MaxN = 10000, Infinity = 1000000000;

struct Edge
{
 int v, l;
 Edge *next;
};

struct Node
{
 int vertex;
 int cost;
};

Edge *g[MaxN + 1];
bool visited[MaxN + 1];
Node d[MaxN + 1];
int n;

bool operator < (Node a, Node b)
{
 return a.cost > b.cost;
}

void addEdge(Edge **head, int v, int l)
{
 Edge *edge = new Edge;
 edge->next = NULL;
 edge->v = v;
 edge->l = l;
 if (*head == NULL)
  *head = edge;
 else
 {
  Edge *tail = *head;
  while (tail->next)
   tail = tail->next;
  tail->next = edge;
 }
}

void deleteEdge(Edge *edge)
{
 if (edge)
 {
  deleteEdge(edge->next);
  delete edge;
 }
}

void dijkstra(int start)
{
 memset(visited, 0, sizeof(visited));
 priority_queue<Node> pq;
 int i;
 for (i = 1; i <= n; i++)
 {
  d[i].vertex = i;
  d[i].cost = Infinity;
 }
 d[start].cost = 0;
 pq.push(d[start]);
 while (!pq.empty())
 {
  Node top = pq.top();
  int v = top.vertex;
  pq.pop();
  if (visited[v])
   continue;
  visited[v] = true;
  Edge *edge = g[v];
  while (edge)
  {
   int k = edge->v;
   int w = edge->l;
   if (d[k].cost > d[v].cost + w)
   {
    d[k].cost = d[v].cost + w;
    pq.push(d[k]);
   }
   edge = edge->next;
  }
 }
}

int main()
{
 freopen("distance.in", "r", stdin);
 freopen("distance.out", "w", stdout);
 int m, x, y, a, b, l, i;
 scanf("%d%d%d%d", &n, &m, &x, &y);
 for (i = 1; i <= n; i++)
  g[i] = NULL;
 for (i = 0; i < m; i++)
 {
  scanf("%d%d%d", &a, &b, &l);
  addEdge(&g[a], b, l);
  addEdge(&g[b], a, l);
 }
 dijkstra(x);
 if (d[y].cost == Infinity)
  printf("-1\n");
 else
  printf("%d\n", d[y].cost);
 for (i = 1; i <= n; i++)
  deleteEdge(g[i]);
 return 0;
}
